﻿using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;
using System;
using System.Linq;
using System.Windows.Forms;


namespace TemperatureSensor_Example {
	public partial class Form1 : Form {
		CommandLineOpen openArgs;
		TemperatureSensor openCh = null;
		formCleaner cleaner;
		TemperatureSensor temp = null;
		private ErrorEventBox errorBox;
		private DataGraphBox tempGraphBox;

		public Form1() {
			InitializeComponent();
			openArgs = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
		}

		public Form1(TemperatureSensor ch) {
			InitializeComponent();
			openCh = ch;
			cleaner = new formCleaner(this);
		}

		private void Form1_Load(object sender, EventArgs e) {
			errorBox = new ErrorEventBox(this);
			tempGraphBox = new DataGraphBox(this, "Temperature");

			this.Size = this.MinimumSize;

			temp = openCh ?? openArgs.makeChannel<TemperatureSensor>();

			//setup event handlers for the phidget
			temp.Attach += temp_attach;
			temp.Detach += temp_detach;
			temp.Error += temp_error;
			temp.TemperatureChange += temp_change;
			temp.PropertyChange += temp_propertychange;

			try {
				temp.Open();
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {

			temp.Attach -= temp_attach;
			temp.Detach -= temp_detach;
			temp.Error -= temp_error;
			temp.TemperatureChange -= temp_change;
			temp.PropertyChange -= temp_propertychange;
			temp.Close();
		}

		void temp_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);

			TemperatureSensor attachedDevice = (TemperatureSensor)sender;

			tempGraphBox.SetTitle(DataGraphBox.GenerateTitle("Temperature", (Phidget)attachedDevice));

			dataIntervalTrk.Unit = "ms";
			dataIntervalTrk.initializeRange(attachedDevice.MinDataInterval, attachedDevice.MaxDataInterval);
			dataIntervalTrk.Value = attachedDevice.DataInterval;

			//differing max/min means the interval is a settable configuration
			if (attachedDevice.MaxDataInterval == attachedDevice.MinDataInterval)
				dataIntervalTrk.Enabled = false;

			changeTriggerTrk.Unit = "°C";
			changeTriggerTrk.minTxt = attachedDevice.MinTemperatureChangeTrigger.ToString();
			changeTriggerTrk.Minimum = attachedDevice.MinTemperatureChangeTrigger;
			changeTriggerTrk.Maximum = attachedDevice.MaxTemperatureChangeTrigger;
			changeTriggerTrk.maxTxt = attachedDevice.MaxTemperatureChangeTrigger.ToString();
			changeTriggerTrk.Value = attachedDevice.TemperatureChangeTrigger;

			//adjust any device specific settings
			switch (temp.ChannelSubclass) {
			case ChannelSubclass.TemperatureSensorThermocouple:
				additionalSettings.Visible = true;
				thermocouplePanel.Visible = true;
				thermocoupletypeCombo.SelectedIndexChanged -= thermocoupletypeCombo_SelectedIndexChanged;
				thermocoupletypeCombo.DataSource = typeof(ThermocoupleType).ToList();
				thermocoupletypeCombo.SelectedValue = attachedDevice.ThermocoupleType;
				thermocoupletypeCombo.SelectedIndexChanged += thermocoupletypeCombo_SelectedIndexChanged;
				break;
			case ChannelSubclass.TemperatureSensorRTD:
				additionalSettings.Visible = true;

				rtdTypePanel.Visible = true;
				rtdTypeCombo.SelectedIndexChanged -= rtdTypeCombo_SelectedIndexChanged;
				rtdTypeCombo.DataSource = typeof(RTDType).ToList();
				rtdTypeCombo.SelectedValue = attachedDevice.RTDType;
				rtdTypeCombo.SelectedIndexChanged += rtdTypeCombo_SelectedIndexChanged;

				rtdWireSetupPanel.Visible = true;
				rtdWireSetupCombo.SelectedIndexChanged -= rtdWireSetupCombo_SelectedIndexChanged;
				rtdWireSetupCombo.DataSource = typeof(RTDWireSetup).ToList();
				rtdWireSetupCombo.SelectedValue = attachedDevice.RTDWireSetup;
				rtdWireSetupCombo.SelectedIndexChanged += rtdWireSetupCombo_SelectedIndexChanged;
				break;
			default:
				break;
			}
			settingsBox.Visible = true;
			outputBox.Visible = true;
		}


		void temp_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			phidgetInfoBox1.Clear();
			cleaner.clean();

			settingsBox.Visible = false;
			outputBox.Visible = false;
			thermocouplePanel.Visible = false;
			rtdTypePanel.Visible = false;
			rtdWireSetupPanel.Visible = false;
			additionalSettings.Visible = false;
			dataIntervalTrk.TrueValue = -1;
		}

		void temp_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);

			if (e.Code == ErrorEventCode.Saturation || e.Code == ErrorEventCode.OutOfRange)
				tempGraphBox.addGap();
		}

		private void temp_propertychange(object sender, PropertyChangeEventArgs e) {

			switch (e.PropertyName) {
			case "DataInterval":
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
					dataIntervalTrk.Value = temp.DataInterval;
					updateGraphDataInterval();
				}
				break;
			case "TemperatureChangeTrigger":
				changeTriggerTrk.Value = temp.TemperatureChangeTrigger;
				break;
			case "DataRate":
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
					dataIntervalTrk.Value = temp.DataRate;
					updateGraphDataInterval();
				}
				break;
			case "RTDWireSetup":
				rtdWireSetupCombo.SelectedValue = temp.RTDWireSetup;
				break;
			}
		}

		void temp_change(object sender, Phidget22.Events.TemperatureSensorTemperatureChangeEventArgs e) {
			tempTxt.Text = e.Temperature.ToString() + "°C";
			tempGraphBox.addData(e.Temperature, temp.DataInterval);
		}

		private void thermocoupletypeCombo_SelectedIndexChanged(object sender, EventArgs e) {
			try {
				temp.ThermocoupleType = (ThermocoupleType)Enum.Parse(typeof(ThermocoupleType), thermocoupletypeCombo.SelectedValue.ToString());
			} catch (PhidgetException ex) { errorBox.addMessage("Error setting thermocouple type: " + ex.Message); }
		}

		private void rtdTypeCombo_SelectedIndexChanged(object sender, EventArgs e) {
			try {
				temp.RTDType = (RTDType)Enum.Parse(typeof(RTDType), rtdTypeCombo.SelectedValue.ToString());
			} catch (PhidgetException ex) { errorBox.addMessage("Error setting RTD type: " + ex.Message); }
		}

		private void rtdWireSetupCombo_SelectedIndexChanged(object sender, EventArgs e) {
			try {
				temp.RTDWireSetup = (RTDWireSetup)Enum.Parse(typeof(RTDWireSetup), rtdWireSetupCombo.SelectedValue.ToString());
			} catch (PhidgetException ex) { errorBox.addMessage("Error setting RTD type: " + ex.Message); }
		}
		private void updateGraphDataInterval() {
			try {
				if (temp.TemperatureChangeTrigger == 0)
					tempGraphBox.DataInterval = (1000.0 / temp.DataRate);
				else
					tempGraphBox.DataInterval = null;
			} catch (PhidgetException ex) { errorBox.addMessage("Error updating data interval: " + ex.Message); }
		}
		private void tempGraphButton_Click(object sender, EventArgs e) {
			if (!tempGraphBox.Visible)
				tempGraphBox.Show();
		}
		private void dataIntervalTrk_KeyOverride(object sender, KeyEventArgs e) {
			e.SuppressKeyPress = true;
			dataIntervalTrk.keyOverride(sender, e);
		}

		private void slider_ValueChangeEvent(object sender, EventArgs e) {
			if (((PhidgetsControlLibrary.CustomScroll)sender).isFixedRange) //datainterval control
			{
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
					try {
						double temporary = dataIntervalTrk.Value;
						temp.DataInterval = (int)temporary;
						if (temp.DataInterval < (int)temporary) {
							while (temp.DataInterval != (int)temporary) {
								temporary++;
								temp.DataInterval = (int)temporary;
							}
						} else {
							while (temp.DataInterval != (int)temporary) {
								temporary--;
								temp.DataInterval = (int)temporary;
							}
						}

						dataIntervalTrk.TrueValue = temp.DataInterval;
						dataIntervalTrk.Value = temp.DataInterval;



						updateGraphDataInterval();
					} catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
				} else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
					try {
						temp.DataRate = dataIntervalTrk.Value;
						updateGraphDataInterval();
					} catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }
				}
			} else //changetrigger control
			  {
				try {
					temp.TemperatureChangeTrigger = changeTriggerTrk.Value;
					updateGraphDataInterval();
				} catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
			}
		}
		private void dataIntervalTrk_ModeSwitch(object sender, EventArgs e) {
			dataIntervalTrk.SwitchModes();
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
				try {
					dataIntervalTrk.isIntValue = false;
					dataIntervalTrk.Unit = "Hz";
					dataIntervalTrk.trkText = temp.DataRate.ToString() + "Hz";
					dataIntervalTrk.Value = temp.DataRate;
				} catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }

			} else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
				try {
					dataIntervalTrk.isIntValue = true;
					if (temp.DeviceID != DeviceID.PN_1046)
						dataIntervalTrk.Unit = "ms";
					dataIntervalTrk.trkText = temp.DataInterval.ToString() + "ms";
					dataIntervalTrk.Value = temp.DataInterval;
				} catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}
	}
}